﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using System.Net;
using System.IO;

using System.Web; //Add reference to System.Web
using System.Runtime.Serialization.Json; //Add reference to System.Runtime.Serialization

namespace RedCritterConnecter.Samples
{
    //Create a class to contain the response

    public class SignInResponse
    {
        public String SessionID { get; set; }
        public String UserID { get; set; }
    
        public Boolean Result { get; set; }
    }


    public class SignIn
    {

        public delegate void OnSignInResponse(SignInResponse SignInResponse);
        public event OnSignInResponse onSignInResponse;

        public delegate void OnSignInResponseError(SignInResponse SignInResponse);
        public event OnSignInResponseError onSignInResponseError;

        const string CONST_ConnecterBaseURL = "https://www.redcritterconnecter.com/";

        public void Execute(String EmailOrUsername, String Password, String InvitationAppDomainID, String InvitationCode, String ShareCode, String GetProfile, String DeveloperLogin, String SessionTimeoutMin)
        {
            try
            {
                //Create url encoded parameters in query string
                String queryString = "emailorusername=" + System.Web.HttpUtility.UrlEncode(EmailOrUsername) + "&password=" + System.Web.HttpUtility.UrlEncode(Password) + "&invitationappdomainid=" + System.Web.HttpUtility.UrlEncode(InvitationAppDomainID) + "&invitationcode='" + System.Web.HttpUtility.UrlEncode(InvitationCode) + "'&sharecode='" + System.Web.HttpUtility.UrlEncode(ShareCode) + "'&getprofile=" + System.Web.HttpUtility.UrlEncode(GetProfile) + "&developerlogin=" + System.Web.HttpUtility.UrlEncode(DeveloperLogin) + "&sessiontimeoutmin=" + System.Web.HttpUtility.UrlEncode(SessionTimeoutMin);

                //Create a new instance of a WebClient
                WebClient wc = new System.Net.WebClient();

                //Prevent this request from caching in order to ensure that it is sent to server
                wc.CachePolicy = new System.Net.Cache.RequestCachePolicy(System.Net.Cache.RequestCacheLevel.NoCacheNoStore);

                //Attach an event handler to receive the response
                wc.DownloadStringCompleted += onSignInResponseReceived;

                //Make the call
                            
                wc.DownloadStringAsync(new Uri(CONST_ConnecterBaseURL + "services/management/SignIn?" + queryString, UriKind.Absolute));
            }
            catch
            {
                //Something went wrong communicating with the server
                if (onSignInResponseError != null)
                {
                    onSignInResponseError(null);
                }
            }

        }

        private void onSignInResponseReceived(object sender, DownloadStringCompletedEventArgs e)
        {
            try
            {
                //Create a JSON serializer
                System.Runtime.Serialization.Json.DataContractJsonSerializer s = new DataContractJsonSerializer(typeof(SignInResponse));
               

                //Copy the string into a memory stream
                MemoryStream ms = new MemoryStream(Encoding.UTF8.GetBytes(e.Result));

                //Read the stream into an object matching the response's type
                SignInResponse SignInResponse = (SignInResponse)s.ReadObject(ms);

                //Work with the populated response object
                if (SignInResponse.Result == true)
                {
                    //Request was accepted, raise the success event
                    onSignInResponse(SignInResponse);
                }
                else
                {
                    //Something went wrong
                    if (onSignInResponseError != null)
                    {
                        //Request failed, raise the error event
                        onSignInResponseError(SignInResponse);
                    }
                }
            }
            catch
            {
                //Something went wrong
                onSignInResponseError(null);
            }
        }
    }
}
